﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;

namespace SimuWAPI
{
    public delegate void HotkeyEventHandler(int HotKeyID);

    public class HotKey : System.Windows.Forms.IMessageFilter
    {
        IntPtr hWnd;
        readonly int WM_HOTKEY = 0x312;
        public enum KeyFlags { MOD_ALT = 0x1, MOD_CONTROL = 0x2, MOD_SHIFT = 0x4, MOD_WIN = 0x8 }

        System.Collections.Hashtable keyIDs = new System.Collections.Hashtable();

        public event HotkeyEventHandler OnHotkey;

        [DllImport("kernel32.dll", SetLastError=true, CharSet=CharSet.Auto)]
        static extern ushort GlobalAddAtom(string lpString);

        [DllImport("kernel32.dll", SetLastError=true, ExactSpelling=true)]
        static extern ushort GlobalDeleteAtom(ushort nAtom);

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool RegisterHotKey(IntPtr hWnd, int id, uint fsModifiers, uint vk);

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool UnregisterHotKey(IntPtr hWnd, int id);

        public HotKey(IntPtr hWnd)
        {
            this.hWnd = hWnd; System.Windows.Forms.Application.AddMessageFilter(this);
        }

        public int RegisterHotkey(System.Windows.Forms.Keys Key, KeyFlags keyflags)
        {
            String keyname = System.Guid.NewGuid().ToString();
            int hotkeyid = GlobalAddAtom(keyname);
            RegisterHotKey((IntPtr)hWnd, hotkeyid, (UInt32)keyflags, (UInt32)Key);
            keyIDs.Add(keyname,hotkeyid);
          
            return hotkeyid;        
        }

        public void UnregisterHotkeys()
        {
            System.Windows.Forms.Application.RemoveMessageFilter(this);
            foreach (int key in keyIDs.Values)           
            {
                UnregisterHotKey(hWnd, key); 
                GlobalDeleteAtom((ushort)key);
            }
        }

        public bool PreFilterMessage(ref System.Windows.Forms.Message m)
        {
            if (m.Msg == WM_HOTKEY)
            {
                if (OnHotkey != null)
                {
                    foreach (int key in keyIDs.Values)
                    {
                        if ((int)m.WParam == key)
                        {
                            OnHotkey((int)m.WParam);
                            return true;
                        }
                    }
                }
            }
            return false;
        }
    }
}
